/* Serial 7 Segment Display Firmware
 Version: 3.0.1
 By: Jim Lindblom (SparkFun Electronics)
 Date: August 24, 2012
 License: This code is beerware: feel free to use it, with or without attribution,
 in your own projects. If you find it helpful, buy me a beer next time you see me
 at the local pub.
 
 Description: This firmware goes on the SparkFun Serial 7-Segment displays.
 https://www.sparkfun.com/search/results?term=serial+7+segment&what=products
 
 You can send the display serial data over either UART, SPI, or I2C. It'll
 sequentially display what it reads. There are special commands to control 
 individual segments, clear the display, reset the cursor, adjust the display's 
 brightness, UART baud rate, i2c address or factory reset.
 
 Note: To use the additional pins, PB6 and PB7, on the ATmega328 we have to add 
 some maps to the pins_arduino.h file. This allows Arduino to identify PB6 and 
 digital pin 22, and PB7 as digital pin 23. Because the Serial 7-Segment runs on 
 the ATmega328's internal oscillator, these two pins open up for our use.
 
 Hardware: You can find the Serial 7-Segment Display Schematic here:
 !!! Add schematic link
 
 */
#include <EEPROM.h>  // Brightness and Baud rate are stored in EEPROM
#include "settings.h"  // Defines command bytes, EEPROM addresses, display data
#include "SevSeg.h" //Library to control generic seven segment displays

SevSeg myDisplay; //Create an instance of the object


//Global variables
unsigned int analogValue6 = 0; //These are used in analog meter mode
unsigned int analogValue7 = 0;
unsigned char deviceMode; // This variable is useds to select which mode the device should be in
unsigned char commandMode = 0;  // Used to indicate if a commandMode byte has been received

// Struct for circular data buffer data received over UART, SPI and I2C are all sent into a single buffer
struct dataBuffer
{
  unsigned char data[BUFFER_SIZE];  // THE data buffer
  unsigned int head;  // store new data at this index
  unsigned int tail;  // read oldest data from this index
} 
buffer;  // our data buffer is creatively named - buffer

// Struct for 4-digit, 7-segment display 
// Stores display value (digits),  decimal status (decimals) for each digit, and cursor for overall display
struct display
{
  char digits[4];
  unsigned char decimals;
  unsigned char cursor;
} 
display;  // displays be displays

void setup()
{  
  setupDisplay(); //Initialize display stuff (common cathode, digits, brightness, etc)

  //We need to check emergency after we have initialized the display so that we can use the display during an emergency reset
  checkEmergencyReset(); //Look to see if the RX pin is being pulled low

  setupTimer();  // Setup timer to control interval reading from buffer
  setupUART();   // initialize UART stuff (interrupts, enable, baud)


  interrupts();  // Turn interrupts on, and les' go

  //Preload the display buffer with a default
  display.digits[0] = 45;
  display.digits[1] = 45;
  display.digits[2] = 45;
  display.digits[3] = 45;
}

// The display is constantly PWM'd in the loop()
void loop()
{
 
  displayData();

}

//This is the normal mode where we display whatever data is coming in over UART
void displayData()
{
  while(1)
  {
    //Just hang out and update the display as new data comes in
    myDisplay.DisplayString(display.digits, display.decimals); //(numberToDisplay, decimal point location)

    serialEvent(); //Check the serial buffer for new data
  }
}



// updateBufferData(): This beast of a function is called by the Timer 1 ISR if there is new data in the buffer. 
// If the data controls display data, that'll be updated.
// If the data relates to a command, commandmode will be set accordingly or a command 
// will be executed from this function.
void updateBufferData()
{

  // First we read from the oldest data in the buffer
  unsigned char c = buffer.data[buffer.tail];
  buffer.tail = (buffer.tail + 1) % BUFFER_SIZE;  // and update the tail to the next oldest

  // if the last byte received wasn't a command byte (commandMode=0)
  // and if the data is displayable (0-0x76 or 0x78), the display will be updated
  if ((commandMode == 0) && ((c < 0x76) || (c == 0x78)))
  {
    display.digits[display.cursor] = c;  // just store the read data into the cursor-active digit
    display.cursor = ((display.cursor + 1) % 4);  // Increment cursor, set back to 0 if necessary
  }
  else if (c == RESET_CMD)  // If the received char is the reset command
  {
    for(int i = 0 ; i < 4 ; i++)
      display.digits[i] = 'x';  // clear all digits
    display.decimals = 0;  // clear all decimals
    display.cursor = 0;  // reset the cursor
  }
  else if (commandMode != 0)  // Otherwise, if data is non-displayable and we're in a commandMode
  {
    switch (commandMode)
    {
    case DECIMAL_CMD:  // Decimal setting mode
      display.decimals = c;  // decimals are set by one byte
      break;
    case BRIGHTNESS_CMD:  // Brightness setting mode
      EEPROM.write(BRIGHTNESS_ADDRESS, c);    // write the new value to EEPROM
      myDisplay.SetBrightness(c); //Set the display to this brightness level
      break;
    case BAUD_CMD:  // Baud setting mode 
      EEPROM.write(BAUD_ADDRESS, c);  // Update EEPROM with new baud setting
      setupUART(); //Checks to see if this baud rate is valid and turns on UART at this speed
      break;
    case CURSOR_CMD:  // Set the cursor
      if (c <= 3)  // Limited error checking, if >3 cursor command will have no effect
        display.cursor = c;  // Update the cursor value
      break;
    case FACTORY_RESET_CMD:  // Factory reset
      setDefaultSettings();  // Reset baud, brightness, and TWI address
      break;
    case DIGIT1_CMD:  // Single-digit control for digit 1
      display.digits[0] = c | 0x80;  // set msb to indicate single digit control mode
      break;
    case DIGIT2_CMD:  // Single-digit control for digit 2
      display.digits[1] = c | 0x80;
      break;
    case DIGIT3_CMD:  // Single-digit control for digit 3
      display.digits[2] = c | 0x80;
      break;
    case DIGIT4_CMD:  // Single-digit control for digit 4
      display.digits[3] = c | 0x80;
      break;
    }
    // Leaving commandMode 
    // !!! If the commandMode isn't a valid command, we'll leave command mode, should be checked below?
    commandMode = 0;
  }
  else  // Finally, if we weren't in command mode, if the byte isn't displayable, we'll enter command mode
  {
    commandMode = c;  // which command mode is reflected by value of commandMode
  }
}

